@@ -1,5 +1,5 @@
 /* Ruby format strings.
-   Copyright (C) 2001-2004, 2006-2009, 2019-2020, 2023 Free Software Foundation, Inc.
+   Copyright (C) 2001-2025 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2020.
 
    This program is free software: you can redistribute it and/or modify
@@ -158,7 +158,8 @@ format_parse (const char *format, bool translated, char *fdi,
   const char *const format_start = format;
   struct spec spec;
   unsigned int unnumbered_arg_count;
-  unsigned int allocated;
+  unsigned int named_allocated;
+  unsigned int numbered_allocated;
   struct spec *result;
 
   spec.directives = 0;
@@ -167,7 +168,8 @@ format_parse (const char *format, bool translated, char *fdi,
   spec.named = NULL;
   spec.numbered = NULL;
   unnumbered_arg_count = 0;
-  allocated = 0;
+  named_allocated = 0;
+  numbered_allocated = 0;
 
   for (; *format != '\0';)
     /* Invariant: spec.numbered_arg_count == 0 || unnumbered_arg_count == 0.  */
@@ -301,13 +303,13 @@ format_parse (const char *format, bool translated, char *fdi,
                     if (seen_precision)
                       {
                         *invalid_reason = INVALID_WIDTH_AFTER_PRECISION (spec.directives);
-                        FDI_SET (format, FMTDIR_ERROR);
+                        FDI_SET (*format == '\0' ? format - 1 : format, FMTDIR_ERROR);
                         goto bad_format;
                       }
                     if (seen_width)
                       {
                         *invalid_reason = INVALID_WIDTH_TWICE (spec.directives);
-                        FDI_SET (format, FMTDIR_ERROR);
+                        FDI_SET (*format == '\0' ? format - 1 : format, FMTDIR_ERROR);
                         goto bad_format;
                       }
                     seen_width = true;
@@ -496,7 +498,7 @@ format_parse (const char *format, bool translated, char *fdi,
                 if (seen_precision)
                   {
                     *invalid_reason = INVALID_PRECISION_TWICE (spec.directives);
-                    FDI_SET (format, FMTDIR_ERROR);
+                    FDI_SET (*format == '\0' ? format - 1 : format, FMTDIR_ERROR);
                     goto bad_format;
                   }
                 seen_precision = true;
@@ -602,10 +604,10 @@ format_parse (const char *format, bool translated, char *fdi,
             /* Register the argument specification for the width.  */
             if (width_number > 0)
               {
-                if (allocated == spec.numbered_arg_count)
+                if (numbered_allocated == spec.numbered_arg_count)
                   {
-                    allocated = 2 * allocated + 1;
-                    spec.numbered = (struct numbered_arg *) xrealloc (spec.numbered, allocated * sizeof (struct numbered_arg));
+                    numbered_allocated = 2 * numbered_allocated + 1;
+                    spec.numbered = (struct numbered_arg *) xrealloc (spec.numbered, numbered_allocated * sizeof (struct numbered_arg));
                   }
                 spec.numbered[spec.numbered_arg_count].number = width_number;
                 spec.numbered[spec.numbered_arg_count].type = FAT_INTEGER;
@@ -613,10 +615,10 @@ format_parse (const char *format, bool translated, char *fdi,
               }
             else if (width_takenext)
               {
-                if (allocated == unnumbered_arg_count)
+                if (numbered_allocated == unnumbered_arg_count)
                   {
-                    allocated = 2 * allocated + 1;
-                    spec.numbered = (struct numbered_arg *) xrealloc (spec.numbered, allocated * sizeof (struct numbered_arg));
+                    numbered_allocated = 2 * numbered_allocated + 1;
+                    spec.numbered = (struct numbered_arg *) xrealloc (spec.numbered, numbered_allocated * sizeof (struct numbered_arg));
                   }
                 spec.numbered[unnumbered_arg_count].number = unnumbered_arg_count + 1;
                 spec.numbered[unnumbered_arg_count].type = FAT_INTEGER;
@@ -629,10 +631,10 @@ format_parse (const char *format, bool translated, char *fdi,
             /* Register the argument specification for the precision.  */
             if (precision_number > 0)
               {
-                if (allocated == spec.numbered_arg_count)
+                if (numbered_allocated == spec.numbered_arg_count)
                   {
-                    allocated = 2 * allocated + 1;
-                    spec.numbered = (struct numbered_arg *) xrealloc (spec.numbered, allocated * sizeof (struct numbered_arg));
+                    numbered_allocated = 2 * numbered_allocated + 1;
+                    spec.numbered = (struct numbered_arg *) xrealloc (spec.numbered, numbered_allocated * sizeof (struct numbered_arg));
                   }
                 spec.numbered[spec.numbered_arg_count].number = precision_number;
                 spec.numbered[spec.numbered_arg_count].type = FAT_INTEGER;
@@ -640,10 +642,10 @@ format_parse (const char *format, bool translated, char *fdi,
               }
             else if (precision_takenext)
               {
-                if (allocated == unnumbered_arg_count)
+                if (numbered_allocated == unnumbered_arg_count)
                   {
-                    allocated = 2 * allocated + 1;
-                    spec.numbered = (struct numbered_arg *) xrealloc (spec.numbered, allocated * sizeof (struct numbered_arg));
+                    numbered_allocated = 2 * numbered_allocated + 1;
+                    spec.numbered = (struct numbered_arg *) xrealloc (spec.numbered, numbered_allocated * sizeof (struct numbered_arg));
                   }
                 spec.numbered[unnumbered_arg_count].number = unnumbered_arg_count + 1;
                 spec.numbered[unnumbered_arg_count].type = FAT_INTEGER;
@@ -656,10 +658,10 @@ format_parse (const char *format, bool translated, char *fdi,
             /* Register the argument specification for the value.  */
             if (name != NULL)
               {
-                if (allocated == spec.named_arg_count)
+                if (named_allocated == spec.named_arg_count)
                   {
-                    allocated = 2 * allocated + 1;
-                    spec.named = (struct named_arg *) xrealloc (spec.named, allocated * sizeof (struct named_arg));
+                    named_allocated = 2 * named_allocated + 1;
+                    spec.named = (struct named_arg *) xrealloc (spec.named, named_allocated * sizeof (struct named_arg));
                   }
                 spec.named[spec.named_arg_count].name = name;
                 spec.named[spec.named_arg_count].type = type;
@@ -667,10 +669,10 @@ format_parse (const char *format, bool translated, char *fdi,
               }
             else if (number > 0)
               {
-                if (allocated == spec.numbered_arg_count)
+                if (numbered_allocated == spec.numbered_arg_count)
                   {
-                    allocated = 2 * allocated + 1;
-                    spec.numbered = (struct numbered_arg *) xrealloc (spec.numbered, allocated * sizeof (struct numbered_arg));
+                    numbered_allocated = 2 * numbered_allocated + 1;
+                    spec.numbered = (struct numbered_arg *) xrealloc (spec.numbered, numbered_allocated * sizeof (struct numbered_arg));
                   }
                 spec.numbered[spec.numbered_arg_count].number = number;
                 spec.numbered[spec.numbered_arg_count].type = type;
@@ -690,10 +692,10 @@ format_parse (const char *format, bool translated, char *fdi,
                     FDI_SET (format, FMTDIR_ERROR);
                     goto bad_format;
                   }
-                if (allocated == unnumbered_arg_count)
+                if (numbered_allocated == unnumbered_arg_count)
                   {
-                    allocated = 2 * allocated + 1;
-                    spec.numbered = (struct numbered_arg *) xrealloc (spec.numbered, allocated * sizeof (struct numbered_arg));
+                    numbered_allocated = 2 * numbered_allocated + 1;
+                    spec.numbered = (struct numbered_arg *) xrealloc (spec.numbered, numbered_allocated * sizeof (struct numbered_arg));
                   }
                 spec.numbered[unnumbered_arg_count].number = unnumbered_arg_count + 1;
                 spec.numbered[unnumbered_arg_count].type = type;
@@ -887,7 +889,7 @@ format_check (void *msgid_descr, void *msgstr_descr, bool equality,
           unsigned int n1 = spec1->named_arg_count;
           unsigned int n2 = spec2->named_arg_count;
 
-          /* Check that the argument names are the same.
+          /* Check the argument names in spec2 are contained in those of spec1.
              Both arrays are sorted.  We search for the first difference.  */
           for (i = 0, j = 0; i < n1 || j < n2; )
             {
